/* $HeadURL$
* $LastChangedRevision$
* $LastChangedDate$
* $LastChangedBy$
*
* Description: Implementierung der Uhr und Alarmfunktionen
* 
* 
*/

/*
Copyright (C) 2012 G. Herzig / DH8GHH

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 3 of the License,
or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, see <http://www.gnu.org/licenses/>
*/

#include <stdlib.h>
#include <stdbool.h>
#include <avr/interrupt.h>
#include "clock.h"
#include "hardware.h"
#include "dcf77.h"
#include "main.h"


// 12.000.000 MHz / 1024 / 100 -> 117,1875
// Da nur 117 bernommen werden kann, wird zu schnell gezhlt und zwar pro interrupt
// 117,1875 / 117 = 1,0016025641025641025641025641026 anstatt 1,0
// 1 / (1,0016025641025641025641025641026 -1) = 624
// Also muss alle 624 Interrupts einmal 10ms weggeworfen werden

#define CLOCKTUNE  624
time_t time;			// Uhrzeit-Struktur

bool time_update;		// Globales Flag ob Uhrzeit aktualisiert werden muss

/**
* Rechnet eine UTC-Stunde auf die aktuelle anzuzeigende Zeitzone um
* d.h. im Falle von UtcMode true, wird die zeit unverndert zurckgegeben
*
* @return Stunde in der gewnschten Zeitzone
*/
static uint8_t utcToLocal(uint8_t utcZeit)
{
	if (Settings.UtcMode == false) // Localzeit
	{
		int8_t utcOffset = Settings.UtcOffset;		// Wert aus RAM in lokale Variable
		// Wenn UTC Offset negativ
		if (utcOffset < 0)
		{
			utcZeit -= utcOffset;	// utcoffset aufaddieren 2mal minux
			if (utcZeit > 23)		// und bei 24h umbrechen
				utcZeit -=24;
		}
		else	// Wenn UTC Offset positiv
		{
			if (utcZeit >= utcOffset)	// Wenn Stunde > Utcoffset
			{
				utcZeit -= utcOffset;	// UtcOffset abziehen
			}
			else	// Wenn utfoffset > stunde
			{
				utcZeit =  24 - utcOffset + utcZeit; // utcoffset an 24h umbrechen
			}
		}
	}
	return utcZeit;
}


/**
* Uhrzeit Stunde erhhen
*/
void SetTimeHour(void)
{
	time_t* ptime = &time;			// Auf die Zeit ber Pointer zugreifen ist effektiver

	if (++ptime->stunde == 24)		// Stundenberlauf bercksichtigen
		ptime->stunde = 0;
}


/**
* Uhrzeit Minute erhhen
*/
void SetTimeMinute(void)
{
	time_t* ptime = &time;			// Auf die Zeit ber Pointer zugreifen ist effektiver

	ptime->sekunde = 0;				// Sekunden auf 0 stellen, wenn die Minute gestellt wird

	if (++ptime->minute == 60)		// Minutenberlauf bercksichtigen. Aber kein Stundenincrement
		ptime->minute = 0;

	
}


/** 
* Uhrzeit aus der time_t Struktur auf dem Display ausgeben.
* Abhngig von der Sekunde und Alarmflags den ":" blinken lassen
*/
void DisplayTime()
{
	time_t* ptime = &time;

	// Ermittle 10er und 1er Stelle Minute
	uint8_t minE = ptime->minute;
	uint8_t minZ = 0;
	while (minE > 9)			// dividieren mit Rest durch schleife. bei max. 60 ist das schneller als div_t
	{
		minE -= 10;
		minZ++;
	}

	// Ermittle 10er und 1er Stelle Stunde
	uint8_t stdE = ptime->stunde;
	stdE = utcToLocal(stdE);	// korrekte Stundenzeit abhngig von UTC ermitteln
	
	uint8_t stdZ = 0;
	while (stdE > 9)			// dividieren mit Rest durch schleife. bei max. 24 ist das schneller als div_t
	{
		stdE -= 10;
		stdZ++;
	}


	//Nu kommt der Wahnsinn mit den Segmenten...
	// Uhr Lokal: 1Hz blinken
	// Uhr UTC  : 0,5 Hz blinken
	// ALARM ein: Unterer Punkt bleibt konstant an
	// ALARM aus: Unterer Punkt blinkt wie der obere
	
	SegDot_t dotOben = SegDot_off;		// Standardmig sind die Dotsegmente AUS
	SegDot_t dotUnten = SegDot_off;		// Standardmig sind die Dotsegmente AUS
	
	uint8_t BlinkMaske = 0b01;			// Blinkmaske 1= 1Hz
	if (Settings.UtcMode)				// Im UTC Mode
	{
		BlinkMaske = 0b10;				// BlinkMaske 2 = 0,5 Hz
	}


	if ((ptime->sekunde & BlinkMaske) == BlinkMaske)	// Wenn Sekunden auf 0b10 oder 0b01 enden (s.o.)
	{
		dotOben = SegDot_on;			// Dotsegmente ein
		dotUnten = SegDot_on;			// Dotsegmente ein
	}

	if (Settings.AlarmMode)				// Wenn Alarm, bleibt der untere Punkt immer an
	{
		dotUnten = SegDot_on;			// dot-Segment unten ein
	}

	// Uhrzeit anzeigen
	WriteSegments(stdZ, stdE, minZ, minE, dotUnten, dotOben);
}


/**
* Alarmzeit Stunde erhhen
* Hier wird mit einer lokalen Variable gearbeitet, da der 2fache zugriff aufs RAM lnger dauern wrde
*/
void SetAlarmHour(void)
{
	uint8_t stunde = Settings.AlarmStd;			// Wert aus RAM in lokale Variable

	if (++stunde == 24)							// Stundenberlauf beachten
		stunde = 0;								// Kein Stundenincrement beim stellen !

	Settings.AlarmStd = stunde;					// lokale Variable schreiben
}


/**
* Alarmzeit Minute erhhen
* Hier wird mit einer lokalen Variable gearbeitet, da der 2fache zugriff aufs RAM lnger dauern wrde
*/
void SetAlarmMinute(void)
{
	uint8_t minute = Settings.AlarmMin;			// Wert aus RAM in lokale Variable

	if (++minute == 60)							// Minutenberlauf beachten
		minute = 0;								// Kein Stundenincrement beim stellen !

	Settings.AlarmMin = minute;					// lokale Variable schreiben
}


/**
* Zeigt den aktuellen AlarmMode auf dem Display an
*/
void DisplayAlarmMode(void)
{
	if (Settings.AlarmMode)	// Alarm true = On 
	{
		WriteSegments(SEG('A','@','O','N'), SegDot_off, SegDot_off);	// Schreibe "A ON"
	}
	else			// Alarm false = Off
	{
		WriteSegments(SEG('A','@','O','F'), SegDot_off, SegDot_off);	// Schreibe "A OF"
	}
}


/**
* Alarmzeit auf dem Display anzeigen
*/
void DisplayAlarmTime(void)
{
	

	// Ermittle 10er und 1er Stelle Minute
	uint8_t minE = Settings.AlarmMin;
	uint8_t minZ = 0;
	while (minE > 9)						// dividieren mit Rest durch schleife. bei max. 60 ist das schneller als div_t
	{
		minE -= 10;
		minZ++;
	}

	// Ermittle 10er und 1er Stelle Stunde
	uint8_t stdE = Settings.AlarmStd;
	uint8_t stdZ = 0;
	stdE = utcToLocal(stdE);			// korrekte Stundenzeit abhngig von UTC ermitteln
	
	while (stdE > 9)					// dividieren mit Rest durch schleife. bei max. 60 ist das schneller als div_t
	{
		stdE -= 10;
		stdZ++;
	}

	WriteSegments(stdZ, stdE, minZ, minE, SegDot_on, SegDot_on);	// Segmente ausgeben
}


// Region UTC

/**
* Incrementiert den UTC-Offset bis max +12h
* Hier wird mit einer lokalen Variable gearbeitet, da der 2fache zugriff aufs RAM lnger dauern wrde
*/
void SetUtcPlus(void)
{
	int8_t utcOffset = Settings.UtcOffset;		// Wert aus RAM in lokale Variable

	if (utcOffset < 12)							// Wenn UTC-Offset kleiner als +12h
	{
		utcOffset++;							// lokale Variable erhhen
		Settings.UtcOffset = utcOffset;			// lokale Variable schreiben
	}
}


/**
* Dekrementiert den UTC-Offset bis max -12h
* Hier wird mit einer lokalen Variable gearbeitet, da der 2fache zugriff aufs RAM lnger dauern wrde
*/
void SetUtcMinus(void)
{
	int8_t utcOffset = Settings.UtcOffset;		// Wert aus RAM in lokale Variable
	if (utcOffset > -12)						// Wenn UTC-Offset grer als -12h
	{
		utcOffset--;							// lokale Variable erniedrigen
		Settings.UtcOffset = utcOffset;			// lokale Variable schreiben
	}
}


/**
* Zeigt den UTC-Offset auf dem Display an
*/
void DisplayUtcOffset(void)
{
	uint8_t signchar = 10;	// stelle Leerzeichen
	int8_t offset = Settings.UtcOffset;			// Wert aus RAM in lokale Variable

	if (offset < 0)	// Wenn UTC-Offset negativ ist
	{
		signchar = 13;	// Vorzeichen "-"
		offset *= -1;	// Kehrwert nehmen
	}

	if (offset >9)	// Wenn Offset > 9, dann eine 1 davor schreiben
	{
		WriteSegments(10,signchar,1, offset-10, SegDot_off, SegDot_off);	// Schreibe " " + Vorzeichen + 1 + rest
	}
	else			// ansonsten Leerzeichen schreiben
	{
		WriteSegments(10,10, signchar, offset, SegDot_off, SegDot_off);		// Schreibe "  " + Vorzeichen + rest
	}
}


/**
* Zeigt den UTC-Mode als Text auf dem Display an
*/
void DisplayUtcMode(void)
{
	if (Settings.UtcMode)
	{
		WriteSegments(SEG('U','T','C','@'), SegDot_off, SegDot_off);		// Schreibe "UTc "
	}
	else
	{
		WriteSegments(SEG('L','O','C','L'), SegDot_off, SegDot_off);		// Schreibe "LocL"
	}
}

/**
* Inrementiert den Ldr-Wert bis max 9
* Hier wird mit einer lokalen Variable gearbeitet, da der 2fache zugriff aufs RAM lnger dauern wrde
*/
void SetLdrPlus(void)
{
	uint8_t ldr = Settings.LDR;			// Wert aus RAM in lokale Variable

	if (ldr < 9)						// Wenn Ldr-Wert kleiner als 9
	{
		ldr++;							// lokale Variable erhhen
		Settings.LDR = ldr;				// lokale Variable schreiben
	}
}


/**
* Dekrementiert den Ldr-Wert bis min 0
* Hier wird mit einer lokalen Variable gearbeitet, da der 2fache zugriff aufs RAM lnger dauern wrde
*/
void SetLdrMinus(void)
{
	uint8_t ldr = Settings.LDR;			// Wert aus RAM in lokale Variable
	if (ldr > 0)						// Wenn Ldr-Wert grer als 0
	{
		ldr--;							// lokale Variable erniedrigen
		Settings.LDR = ldr;				// lokale Variable schreiben
	}
}


/**
* Zeigt den Ldr-Wert auf dem Display an
*/
void DisplayLdrValue(void)
{
	uint8_t ldr = Settings.LDR;

	if (ldr == 0)						// LDR0 = automatik
	{
		WriteSegments(SEG('A','U','T','O'), SegDot_off, SegDot_off);
	}
	else
	{
		WriteSegments(SEG('L','@','@',(ldr+48)), SegDot_off, SegDot_off);
	}
}




/**
* Zeigt das Datum auf dem Display an und nutzt den unteren Dot als Tag/Monat Trenner
*/
void DisplayDate(void)
{
	time_t* ptime = &time;		// Auf die Alarmzeit ber Pointer zugreifen ist effektiver

	// Ermittle 10er und 1er Stelle Tag
	uint8_t tagE = ptime->day;
	uint8_t tagZ = 0;
	while (tagE > 9)						// dividieren mit Rest durch schleife. bei max. 31 ist das schneller als div_t
	{
		tagE -= 10;
		tagZ++;
	}

	// Ermittle 10er und 1er Stelle Monat
	uint8_t monE = ptime->month;
	uint8_t monZ = 0;
	
	while (monE > 9)					// dividieren mit Rest durch schleife. bei max. 60 ist das schneller als div_t
	{
		monE -= 10;
		monZ++;
	}

	WriteSegments(tagZ, tagE, monZ, monE, SegDot_on, SegDot_off);			// Segmente ausgeben
}


/**
* Zeigt den Sync-Sound Mode als Text auf dem Display an
*/
void DisplaySyncSndMode(void)
{
	if (Settings.SyncSound)
	{
		WriteSegments(SEG('B','@','O','N'), SegDot_off, SegDot_off);		// Schreibe "B ON"
	}
	else
	{
		WriteSegments(SEG('B','@','O','F'), SegDot_off, SegDot_off);		// Schreibe "B OF"
	}
}



/** Prft, ob die alarmzeit erreicht ist, mit einer Toleranz von 5 Sekunden
*   Die Funktion kann geinlined werden, da sie nur einmal aufgerufen wird
*
* @return true wenn Alarmzeit = Uhrzeit
*/
static inline bool isTimeToPiep(void)
{
	Settings_t* pSettings = &Settings;	// MitmPointer rechnets sich effektiver bei mehreren Zugriffen, sofern man die Struct-Reihenfolge einhlt
	time_t * pTime = &time;				// Pointer (damit rechnets besser) auf Zeit holen

	if ((pSettings->AlarmMode == true)				// Wenn Alarm aktiv
		&& (pSettings->AlarmStd == pTime->stunde)	// und Stunde stimmt
		&& (pSettings->AlarmMin == pTime->minute)	// und Minute stimmt
		&& (pTime->sekunde < 5))					// und Minute grad angefangen hat
	{
		return true;	// is Time to Piep ;)
	}
	return false;		// Alarm nicht erreicht
}


/**
* Handled alle Uhrzeit-relevanten Funktionen die im 10ms Takt erledigt werden mssen
*/
static inline void Clock_10ms(void)
{
	// Zeiger auf die Zeit ermglicht effektiven Zugriff
	time_t* ptime = &time;

	if ((++ptime->msec) == 100)		// 1 Sekunde voll ?
	{
		ptime->msec = 0;
		time_update = 1;			// Zeit aktualisieren (im Sekundentakt, damits blinkt ;) 
		
		if((++ptime->sekunde) == 60)	// 1 Minute voll ?
		{
			ptime->sekunde = 0;
			ptime->minute++;		// raus gezogen, weil sonst "timetoPiep" nicht die korrekte Zeit kennt

			if (isTimeToPiep())		// Prfen ob Alarmzeit erreicht wurde
			{
				SetHupe(true);		// Alarm aktivieren
			}
			if((ptime->minute) == 60)	// Stunde voll ?
			{
				ptime->minute = 0;
				if((++ptime->stunde) == 24)	// Tag voll ?
				{
					ptime->stunde = 0;
				}
			}	
		}
	}
}


/**
* Pinwackeln als debug-Ausgabe
* Inline funktion, da es sich fr ein Befehl nicht lohnt, 3 Assembleranweisungen fr nen Funktionsaufruf zu opfern
*/
static inline void TogglePin(void)
{
	PIND |= (1<<PD6);	// Pin PD6 Toggeln
}


/**
* Interrupt Service Routine Timer2 Compare Match
* Wird alle Quarz / Vorteiler / 100 = 10ms aufgerufen und verarbeitet alle zyklischen Prozesse
*/
ISR (TIMER2_COMPA_vect)
{
	static uint16_t clocktune;

	TogglePin();				// Pinwackeln als Debug-Ausgabe
	job77_dcf_10ms();			// DCF77 Pegel parsen
	if (++clocktune < CLOCKTUNE)
	{
		Clock_10ms();				// Uhrzeit incrementieren
	}
	else
	{
		clocktune = 0;
	}
	ReadKeyFromHardware();		// Tasten einlesen
	DoHupe();					// Hupen, falls notwendig
}