/* $HeadURL$
* $LastChangedRevision$
* $LastChangedDate$
* $LastChangedBy$
*
* Description: Deklarationen zum Thema DCF77 Sampling und Auswertung
* Code mit freundlicher Genehmigung von Georg-Johann Lay
* http://www.gjlay.de/software/c-code/dcf77.html
* * 
* 
*/

/*
Copyright (C) 20?? Georg-Johann Lay 
Extensions in 2012 by G. Herzig / DH8GHH

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 3 of the License,
or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, see <http://www.gnu.org/licenses/>
*/

#ifndef DCF77_H
#define DCF77_H

#include <stdint.h>
#include "clock.h"

#define TIME_MESZ	1
typedef struct
{
    // In newtime wird die neue Zeit-Information aufgebaut
    time_t newtime;

    // Bei korrekt empfangener Zeit wird die Zeit nach *ptime kopiert.
    //
    // !!! *ptime MUSS AUSSERHALB DES DCF77-MODULS GESETZT WERDEN  !!!
    // !!! ANSONSTEN WIRD DIE ZEIT NICHT HINEINKOPIERT             !!!
    time_t *ptime;

    // Wenn die neue Zeit nach *ptime kopiert wird, wird time_changed
    // auf 1 gesetzt. Die Anwendung kann time_changed wieder auf
    // 0 zurcksetzten. So kann die Anwendung erkennen, da ein neues
    // Zeit/Datums-Paket angekommen ist.
    uint8_t time_changed;

    // Fr interne Verwendung in dcf77.c

    // Parity wird on the fly mitberechnet
    uint8_t parity;

    // Es gab einen Fehler: error != 0
    uint8_t error;

    // Wertigkeit des aktuellen BCD-Bits: 1, 2, 4, 8, 10, 20, 40, 80.
    uint8_t bitval;

    // Dauer der aktuellen Sekunde in Einheiten von 10ms (1 Tick)
    uint8_t ticks;

    // Letzter poll-Wert vom Aufruf von dcf77_tick() zur
    // Flankenerkennung.
    uint8_t poll;

    uint8_t byteno;
} dcf77_t;

// dcf77 wird definiert in dcf77.c
extern dcf77_t dcf77;

/* Obwohl es nur eine dcf77-Struktur gibt, wird nicht direkt
   mittels dcf77.xxx darauf zugegriffen, sondern durch dcf->xxx.
   Grund ist, da auf AVR durch indirekten Zugriff ein krzerer
   Code mglich ist als durch direkten Zugriff. Dazu darf gcc die
   Adresse von dcf77 nicht zu sehen bekommen; ansonsten macht er
   auch bei dcf->xxx einen direkten Zugriff. */

extern void dcf77_tick (dcf77_t *dcf, uint8_t poll);
void job77_dcf_10ms (void);
void CheckDCF77Sync(void);
bool IsDCFSync(void);

#endif // DCF77_H