/* $HeadURL$
* $LastChangedRevision$
* $LastChangedDate$
* $LastChangedBy$
*
* Description: Deklarationen zum ThemaHardwareansteuerung
* Display, Tasten, LDR und PWM
* 
* 
*/

/*
Copyright (C) 2012 G. Herzig / DH8GHH

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 3 of the License,
or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, see <http://www.gnu.org/licenses/>
*/

#ifndef HARDWARE_H
#define HARDWARE_H

#include <stdbool.h>
#include <avr/io.h>

// Macro zur Wiedergabe von Text. Die 4 Buchstaben werden um 48 erniedrigt gem ASCII -> Position in Symboltabelle
#define SEG(w,x,y,z)	w-48,x-48,y-48,z-48

// Datentyp fr die Dots
typedef enum
{
	SegDot_on = 0b10000000,		// entspricht Bitmaske um nen Dot leuchten zu lassen
	SegDot_off= 0b00000000		// entspricht Bitmakse fr dot dunkel
} SegDot_t;


// Extra Name fr den Port auf dem die Tasten eingelesen werden
#define TAST_IO	PIND

// Bitmaskendefinitionen fr die Tasten
#define TAST_1	(1<<PD0)
#define TAST_2	(1<<PD3)
#define TAST_3	(1<<PD4)
#define TAST_4	(1<<PD1)
#define TAST_5	(1<<PD7)

// Bitmaskendefinition fr "alle Tasten"
#define TAST_MASK	(TAST_1 | TAST_2 | TAST_3 | TAST_4 | TAST_5)

// typisiertes Enum fr die Tasten, erhht die Typsicherheit
typedef enum
{
	Tast_None	= 0,
	Tast_Alarm	= TAST_5,
	Tast_Disp	= TAST_1,
	Tast_Set	= TAST_2,
	Tast_Plus	= TAST_3,
	Tast_Minus	= TAST_4,
} Tast_t;


void SetHupe(bool Hupestatus);		// Hupe ein oder ausschalten
bool GetHupe(void);					// Status der Hupe abfragen
void DoHupe(void);					// Workerfunktion, die die Hupe zyklisch hupen lsst


void ReadKeyFromHardware(void);		// Liest Keys in Buffer ein
const Tast_t GetKey(void);			// Ausgabe Keyboard Buffer

void AutoAdjustLED(void);			// Justiert LED-PWM

void WriteSegments(uint8_t a, uint8_t b, uint8_t c, uint8_t d, SegDot_t dotUnten, SegDot_t dotOben);
void DisplaySplash(void);			// Zeigt "FIFI Uhr"
void DisplayVersion(void);			// Zeigt Revision Nummer

void SetBlinkMode(bool Mode);		// Aktiviert den BlinkMode
bool isDisplayOn(void);				// Workerfunktion die prft, ob das Display im Status "an" oder "aus" ist

#endif // HARDWARE_H