/* $HeadURL$
* $LastChangedRevision$
* $LastChangedDate$
* $LastChangedBy$
*
* Description: Hauptprogramm der Uhr. Initialisiert Hardware und enthlt main-loop
* 
* 
*/

/*
Copyright (C) 2011 Radig Ulrich  mailto: mail@ulrichradig.de
Extensions in 2012 by G. Herzig / DH8GHH

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 3 of the License,
or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, see <http://www.gnu.org/licenses/>
*/

#include <stdlib.h>
#include <stdbool.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include "main.h"
#include "hardware.h"
#include "clock.h"
#include <util/delay.h>
#include "dcf77.h"

/**
* Initialisiert smtliche Hardware
*/
static inline void Init(void)
{
	// Init Ports
	DDRB = (1<<PB5)		// Output SPI SCK
		 | (0<<PB4)		// Input SPI MISO
		 | (1<<PB3)		// Output SPI MOSI
		 | (1<<PB2)		// Output SPI Segments OE  
		 | (1<<PB1)		// Output PWM
		 | (1<<PB0);	// Output DCF77 Enable

	PORTB = (0<<PB5)	// Low SPI Idle
		  | (0<<PB4)	// HighZ SPI Idle
		  | (0<<PB3)	// Low SPI Idle
		  | (1<<PB2)	// High LED Output disable (wenn PWM aus, werden die LEDs disabled)
		  | (0<<PB1)	// Low PWM Off
		  | (1<<PB0);	// Low DCF77Rx Enable


	DDRC = (0<<PC6)		// Input / Unbenutzt
		 | (0<<PC5)		// Input / Unbenutzt
		 | (0<<PC4)		// Input / Unbenutzt
		 | (0<<PC3)		// Input / Unbenutzt
		 | (0<<PC2)		// Input / Unbenutzt
		 | (0<<PC1)		// Input / Unbenutzt
		 | (0<<PC0);	// Input / Unbenutzt

	PORTC = (0<<PC6)	// HighZ / Unbenutzt
		  | (0<<PC5)	// HighZ / Unbenutzt
		  | (0<<PC4)	// HighZ / Unbenutzt
		  | (0<<PC3)	// HighZ / Unbenutzt
		  | (0<<PC2)	// HighZ / Unbenutzt
		  | (0<<PC1)	// HighZ / Unbenutzt
		  | (0<<PC0);	// HighZ / Unbenutzt


	DDRD = (0<<PD7)		// Input Taster 5
		 | (1<<PD6)		// Output / Tuningsignal
		 | (1<<PD5)		// Output Hupe
		 | (0<<PD4)		// Input Taster 3
		 | (0<<PD3)		// Input Taster 2
		 | (0<<PD2)		// Input DCF77 Signal
		 | (0<<PD1)		// Input Taster 4
		 | (0<<PD0);	// Input Taster 1

	PORTD = (1<<PD7)	// PullUp Taster 5
		  | (0<<PD6)	// Low Tuningsignal
		  | (0<<PD5)	// Low Hupe aus
		  | (1<<PD4)	// PullUp Taster 3
		  | (1<<PD3)	// PullUp Taster 2
		  | (0<<PD2)	// HighZ fr DCF Modul
		  | (1<<PD1)	// PullUp Taster 4
		  | (1<<PD0);	// PullUp Taster 1

	//Init SPI Schnittstelle (S.168)
	SPCR = (0<<SPIE)	// disable SPI Interrupt
		 | (1<<SPE)		// enable SPI
		 | (0<<DORD)	// MSB first
		 | (1<<MSTR)	// Enable Master
		 | (0<<CPOL)	// normal Clock Polarity
		 | (0<<CPHA)	// Normal Clock Phase
		 | (0<<SPR1)	// Clock Speed Full
		 | (0<<SPR0);	// Clock Speed Full

	SPCR |= (1<<SPI2X);	// SPI clock doublespeed

	// Init Timer1 : 12,8V und LED-PWM
	TCCR1A = (1<<COM1A1)	// Clear OC on Compare Match 
		   | (0<<COM1A0)	// and Set OC on Bottom
		   | (1<<COM1B1)	// Clear OC on Compare Math
		   | (0<<COM1B0)	// and Set OC on Bottom
		   | (1<<WGM11)		
		   | (0<<WGM10);	// PWM mit benutzerdef. TOP (ICR1) 

	TCCR1B = (0<<ICNC1)		// Input Capture Noise canceler off
		   | (0<<ICES1)		// Input Capture edge select -> egal
		   | (1<<WGM13)		// siehe WGM10
		   | (1<<WGM12)		// siehe WGM10
		   | (0<<CS12)		
		   | (0<<CS11)
		   | (1<<CS10);		// Clock/1 

	ICR1 = 200;				// Top-Value fr PWM
	OCR1B = 0;				// Setze LED auf hellste stufe


	// Init Timer 2 : 10ms Taktung
	
	TCCR2A = (0<<COM2A1)	
		   | (0<<COM2A0)	// Kein Compare Match Output A
		   | (0<<COM2B1)
		   | (0<<COM2B0)	// Kein Compare Match Output B
		   | (1<<WGM21)		// Siehe WGM22
		   | (0<<WGM20);	// Siehe WGM22

	TCCR2B = (0<<FOC2A)		// Kein Compare Match A forced
		   | (0<<FOC2B)		// Kein Compare Match A forced
		   | (0<<WGM22)		// Mode 2: CTC
		   | (1<<CS22)
		   | (1<<CS21)
		   | (1<<CS20);		// Vorteiler /128 (und 8)


	OCR2A = F_CPU / 1024 / 100 -1  ;	// 10ms Takt.

	TIMSK2 = (0<<OCIE2B)
		   | (1<<OCIE2A)	// Enable Compare Match interrupt
		   | (0<<TOIE2);


	// Init ADC (Seite 257)

	ADMUX = (0<<REFS1)
		  | (1<<REFS0)		// Referenc: AVcc
		  | (0<<ADLAR)		// Right Adjusted
		  | (0<<MUX3)
		  | (0<<MUX2)
		  | (0<<MUX1)
		  | (0<<MUX0);		// Input 0

	ADCSRA = (1<<ADEN)		// Enable
		   | (1<<ADSC)		// Start Conversion
		   | (1<<ADATE)		// Auto Trigger Enable
		   | (0<<ADIF)		// Unbenutzt: InterruptFlag
		   | (1<<ADIE)		// Enable Interrupt
		   | (1<<ADPS2)
		   | (1<<ADPS1)
		   | (1<<ADPS0);	// Prescaler 128

	ADCSRB = (0<<ACME)		// Analog Comparator Mux disable
		   | (0<<ADTS2)
		   | (0<<ADTS1)
		   | (0<<ADTS0);	// ADC AutoTriggerSource: Free running mode

	PORTB &= ~(1<<PB0);		// PB0 to Low -> DCF77 RX aktivieren
}


static Settings_t EEMEM EE_Settings;	// Reservierung fr EEPROM-Speicherblock
 Settings_t Settings;					// Einstellungsblock im RAM

/**
* Ldt Alarm und UTC Einstellungen aus dem EEPROM
* Falls die Versionskennung nicht passt, startet die Uhr mit Default-Werten
*/
static void LoadSettings(void)
{
	eeprom_read_block(&Settings, &EE_Settings, sizeof(Settings));	// Settings Block aus EEPROM laden

	if (Settings.Version != SETTINGS_VERSION)		// Wenn Version nicht passt, Defaultwerte setzen
	{
		Settings.AlarmMode = true;		// Alarm ein
		Settings.AlarmStd = 18;			// 18:15 UTC -> Simpsons ;)
		Settings.AlarmMin = 15;
		Settings.UtcOffset = -2;		// UTC-Offset -2h
		Settings.UtcMode= false;		// LocalTime
		Settings.LDR = 0;
		Settings.SyncSound = true;
	}
}

/**
* Speichert Alarm und UTC Einstellungen im EEPROM
*/
static void SaveSettings(void)
{
	Settings.Version = SETTINGS_VERSION;		// Settings-Versions-Nummer setzen. Wird gebraucht um nderungen in der Struktur zu erkennen

	eeprom_write_block(&Settings, &EE_Settings, sizeof(Settings));	// Settings-Block im EEPROM speichern
}


static MenuMode_t menuMode;		// Merker fr den Modus Set/Alarm/None

#define MAXSETMODE 4

/**
* Liest die gedrckte Taste ein und fhrt die entsprechende Funktion aus.
* Die Tasten Alarm und Set aktivieren den entsprechenden MenMode.
* Innerhalb der MenModes werden die Tastendrcke verarbeitet
*/
void HandleTasten(void)
{
	static uint8_t setMode;							// Merker fr den Submenmodus innerhalb SET

	Tast_t currentkey = GetKey();

	/// im Ruhezustand reagieren wir auf Alarm, Set, Display
	if (MenuMode_None == menuMode)
	{
		if (Tast_Alarm == currentkey)				// Taste Alarm ist gedrckt
		{
			if (GetHupe()== true)
			{
				SetHupe(false);						// Falls ein Alarm an war, ist er nun aus
			}
			else
			{
				menuMode = MenuMode_Alarm;			// MenMode setzen
				SetBlinkMode(true);					// rumblinken
				DisplayAlarmTime();					// Alarmzeit anzeigen
			}
			while (GetKey() == currentkey);			// warten auf Taste(n) losgelassen
		}

		if (Tast_Set == currentkey)
		{
			menuMode = MenuMode_Set;				// MenMode setzen fr nachfolgende Tastendrucks
			SetBlinkMode(true);						// rumblinken

			switch(setMode)							// aktuellen (bzw grad genderten) SetMode anzeigen
			{
			case 0: WriteSegments(SEG('U','H','R','@'), SegDot_off, SegDot_off); break;
			case 1: WriteSegments(SEG('O','F','F','S'), SegDot_off, SegDot_off); break;
			case 2: WriteSegments(SEG('Z','O','N','E'), SegDot_off, SegDot_off); break;
			case 3: WriteSegments(SEG('L','D','R','@'), SegDot_off, SegDot_off); break;	
			case 4: WriteSegments(SEG('S','Y','N','C'), SegDot_off, SegDot_off); break;
			}
			while(GetKey() == currentkey);			// warten auf Taste(n) losgelassen
		}

		if (Tast_Disp == currentkey)
		{
			menuMode = MenuMode_Disp;				// Menmode setzen
			DisplayDate();							// Datum anzeigen
			while (GetKey() == currentkey);			// warten auf Taste(n) losgelassen
		}

		return;	// hier raus, sonst werden die anderen tasten gehandled...
	}
	//// Ende Ruhezustand

	//// Im Alarm-Men reagieren wir auf Alarm,Set,Plus, Minus
	if (MenuMode_Alarm == menuMode)
	{
		if (Tast_Alarm == currentkey)				// Taste Alarm verarbeiten
		{
			menuMode = MenuMode_None;				// Wieder aus Alarm Mode raus
			SetBlinkMode(false);					// aufhren zu blinken
			SaveSettings();							// Einstellungen speichern

			while(GetKey() == currentkey);			// warten auf Taste(n) losgelassen
		} // ende Taste Alarm


		if (Tast_Set == currentkey)					// Taste Set verarbeiten
		{
			Settings.AlarmMode = !Settings.AlarmMode;// Alarmfunktion de/aktivieren
			DisplayAlarmMode();						// AlarmMode anzeigen

			while (GetKey() == currentkey);			// warten auf Taste(n) losgelassen
		} // ende Taste Set

		if (Tast_Plus == currentkey)				// Taste Plus verarbeiten
		{
			SetAlarmHour();							// Alarmzeit-Stunde stellen
			DisplayAlarmTime();						// Alarmzeit anzeigen

			while (GetKey() == currentkey);			// warten auf Taste(n) losgelassen
		} // ende Taste Plus

		if (Tast_Minus == currentkey)				// Taste Minus verarbeiten
		{
			SetAlarmMinute();						// Alarmzeit-Minute stellen
			DisplayAlarmTime();						// Alarmzeit anzeigen

			while (GetKey() == currentkey);			// warten auf Taste(n) losgelassen
		} // ende Taste Minus

		return;	// hier raus, sonst werden die anderen tasten gehandled...
	}
	//// ende Alarm-Men

	//// Im Set-Men reagieren wir nur auf die Display-Taste
	if (MenuMode_Set == menuMode)
	{
		if (Tast_Set == currentkey)					// Taste Set verarbeiten
		{
			menuMode = MenuMode_SetItem;			// Menmode wechseln, da wir jetzt eine Ebene tiefer sind.

			switch(setMode)							// Abhngig vom gewhlten SetMode die richtige Aktion ausfhren
			{	
			case 0:									// Im Set Uhr mode
				DisplayTime();						// Uhrzeit anzeigen
				break;
			case 1:									// Im Set Offset mode
				DisplayUtcOffset();					// UTC-Offset anzeigen
				break;
			case 2:									// Im Set Zone mode
				DisplayUtcMode();					// Gewhlte Zone anzeigen
				break;
			case 3:									// Im Set LDR mode
				DisplayLdrValue();					// LDR-Wert anzeigen
				break;
			case 4:									// Im Set SyncSound mode
				DisplaySyncSndMode();				// Gewhlten Mdus anzeigen
				break;
			}

			while(GetKey() == currentkey);			// warten auf Taste(n) losgelassen
		}

		if (Tast_Disp == currentkey)				// Taste Display verarbeiten
		{
			if (++setMode > MAXSETMODE)				// SetMode incrementieren und bei MAXSETMODE von vorn beginnen
			{
				setMode = 0;
			}

			// aktuellen (bzw grad genderten) SetMode anzeigen
			switch(setMode)
			{
			case 0: WriteSegments(SEG('U','H','R','@'), SegDot_off, SegDot_off); break;
			case 1: WriteSegments(SEG('O','F','F','S'), SegDot_off, SegDot_off); break;
			case 2: WriteSegments(SEG('Z','O','N','E'), SegDot_off, SegDot_off); break;
			case 3: WriteSegments(SEG('L','D','R','@'), SegDot_off, SegDot_off); break;	
			case 4: WriteSegments(SEG('S','Y','N','C'), SegDot_off, SegDot_off); break;
			}

			while(GetKey() == currentkey);			// warten auf Taste(n) losgelassen
		} // ende Display

		return;	// hier raus, sonst werden die anderen tasten gehandled...
	}
	//// Ende Set Mode

	//// Im Set-Items-Men reagieren wir auf Set,Display,Plus,Minus
	if (MenuMode_SetItem == menuMode)
	{
		if (Tast_Set == currentkey)					// Taste Set verarbeiten
		{
			menuMode = MenuMode_None;				// Wieder aus Set Mode raus
			SetBlinkMode(false);					// aufhren zu blinken
			SaveSettings();							// Einstellungen speichern

			while(GetKey() == currentkey);			// warten auf Taste(n) losgelassen
		} // ende Set


		if (Tast_Plus == currentkey)				// Taste Plus verarbeiten
		{
			switch(setMode)							// Abhngig vom gewhlten SetMode die richtige Aktion ausfhren
			{	
			case 0:									// Im Set Uhr mode
				SetTimeHour();						// Uhrzeit Stunde stellen
				DisplayTime();						// Uhrzeit anzeigen
				break;
			case 1:									// Im Set Offset mode
				SetUtcPlus();						// UTC-Offset erhhen
				DisplayUtcOffset();					// UTC-Offset anzeigen
				break;
			case 2:									// Im Set Zone mode
				Settings.UtcMode = false;			// LOCAL auswhlen					
				DisplayUtcMode();					// Gewhlte Zone anzeigen
				break;
			case 3:									// Im Set LDR mode
				SetLdrPlus();						// LDR Wert vergrern 
				DisplayLdrValue();					// LDR Wert anzeigen
				break;
			case 4:									// Im Set SyncSound mode
				Settings.SyncSound = true;			// SyncSound einschalten
				DisplaySyncSndMode();				// Gewhlten Modus anzeigen
				break;
			}
			while(GetKey() == currentkey);			// warten auf Taste(n) losgelassen
		} // ende Plus

		if (Tast_Minus == currentkey)				// Taste Minus verarbeiten
		{
			switch(setMode)
			{
			case 0:									// Im Set Uhr mode
				SetTimeMinute();					// Uhrzeit Minute stellen
				DisplayTime();						// Uhrzeit anzeigen
				break;
			case 1:									// Im Set Offset mode
				SetUtcMinus();						// UTC-Offset verringern
				DisplayUtcOffset();					// UTC-Offset anzeigen
				break;
			case 2:									// Im Set Zone mode
				Settings.UtcMode = true;			// UTC auswhlen
				DisplayUtcMode();					// Gewhlte Zone anzeigen
				break;
			case 3:									// Im Set LDR mode
				SetLdrMinus();						// LDR Wert verkleinern
				DisplayLdrValue();					// LDR Wert anzeigen
				break;
			case 4:									// Im Set SyncSound mode
				Settings.SyncSound = false;			// SyncSound ausschalten
				DisplaySyncSndMode();				// Gewhlten Modus anzeigen
				break;
			}
			while(GetKey() == currentkey);			// warten auf Taste(n) losgelassen
		} // Ende Minus

		return;	// hier raus, sonst werden die anderen tasten gehandled...
	} //// Ende Set-Items-Men

	//// Im Display Men reagieren wir auf Taste Display -> fhrt wieder raus
	if (MenuMode_Disp == menuMode)
	{
		if (Tast_Disp == currentkey)				// Taste DISP verarbeiten
		{
			menuMode = MenuMode_None;				// Wieder aus dem Men raus

			while(GetKey() == currentkey);			// warten auf Taste(n) losgelassen
		}
	} /// Ende Display-Men
}



/**
* Hauptprogramm
* Initialisiert die Hardware, Ldt Einstellungen und Zeigt den Splashscreen an.
* Anschlieend werden in einer Endloschleife Helligkeit, Tasten, DCF77 und Uhrzeit verarbeitet
*/
int main (void) 
{
	Init();		// Initialisiere Hardware
	
	OCR1BL = 156;//133 + (240 - ldr_value);

	if (((~TAST_IO) & Tast_Set) == Tast_Set)
		DisplayVersion();

	dcf77.ptime = &time;	// Zeitpointer zum synchen zuweisen
	dcf77.error = 1;		// erstmal ist DCF error
	LoadSettings();			// Settings laden

	time.stunde = 0;		// Uhr startet bei 0 Uhr

	if (Settings.UtcMode == false)				// Wenn eine LocalZone gewhlt wurde
	{
		time.stunde = 24 + Settings.UtcOffset;	// UTC-Zeit anpassen, damit LocalZone 0 Uhr anzeigt
		if (time.stunde > 23)					// Auf einen 24h berlauf reagieren
		{
			time.stunde -= 24;
		}
	}

	sei();					// Globale Interrupts Enable, ab jetzt luft Zeit, Tasten, Helligkeitsregelung
	DisplaySplash();		// Begrungstext
	DisplayTime();			// Zeit anzeigen

	while(1)				// Hauptschleife fr Uhrbetrieb
	{
		AutoAdjustLED();	// LED Helligkeit justieren
		HandleTasten();		// Tasten abfragen
		CheckDCF77Sync();	// DCF77 Leitung abfragen

		if (MenuMode_None == menuMode)	// Wenn im normalen Uhrenbetrieb
		{
			if(time_update)				// Wenn anzeige aktualisiert werden muss
			{
				DisplayTime();			// Zeit anzeigen
				time_update = 0;		// Flag zurcksetzen
			}
		}
	}
}

